import argparse
from gpuc.core.task.tstructure import TStrucutre
from gpuc.core.task.tpeople import TPeople, Population
from gpuc.lib.mlogger import BufferingSMTPHandler
from gpuc.config import MAIL_SETTINGS

import logging


def _init_logging(*, verbose: bool, sftp_enabled: bool):
    """Initialise la configuration du logging.

    Args:
        verbose (bool): Indique si les logs détaillés doivent être affichés dans la console.
        sftp_enabled (bool): Indique si l'envoi par SFTP est activé.
    """
    handlers = (
        [logging.StreamHandler()] if verbose else []
    )  # Affichage les logs dans le terminal si verbose est activé

    if sftp_enabled:
        smtp_handler = BufferingSMTPHandler(
            mailhost=MAIL_SETTINGS.host,
            mailport=MAIL_SETTINGS.port,
            mailauth=MAIL_SETTINGS.login,
            mailpwd=MAIL_SETTINGS.password,
            fromaddr=MAIL_SETTINGS.sender,
            toaddrs=MAIL_SETTINGS.receivers,
            subject="[GPUC][Notification] Synchro autorisation ",
            capacity=100,
            tls=False,
        )
        handlers.append(smtp_handler)

    logging.basicConfig(
        format="%(asctime)s :: %(name)s :: %(levelname)s :: %(message)s",
        level=logging.INFO,
        handlers=handlers,
    )


def main():
    parser = argparse.ArgumentParser(
        description="Outil CLI pour gérer les fichiers GPUC.",
        epilog="Exemples d'utilisation :\n"
        "  python cli.py people generate --mode PERSONNEL_ONLY\n"
        "  python cli.py structure generate --sftp",
        formatter_class=argparse.RawTextHelpFormatter,
    )
    parser.add_argument(
        "--verbose",
        action="store_true",
        help="Activer les logs détaillés dans la console.",
    )
    subparsers = parser.add_subparsers(dest="command", required=True)

    # Subparser pour les employés (people)
    people_parser = subparsers.add_parser(
        "people",
        help="Gestion des autorisations.",
        description="Permet de générer le fichier d'autorisation liés aux employés et/ou aux étudiants.",
        formatter_class=argparse.RawTextHelpFormatter,
    )
    people_parser.add_argument(
        "action",
        choices=["generate"],
        help="Génération des fichiers d'autorisation pour les employés et/ou les étudiants.",
    )
    people_parser.add_argument(
        "--population",
        type=Population,
        choices=list(Population),
        help=(
            "Mode de génération des fichiers :\n"
            "  - PERSONNEL_ONLY : Génère uniquement les données des employés.\n"
            "  - STUDENT_ONLY : Génère uniquement les données des étudiants.\n"
            "  - ALL : Génère les données des employés et des structures associées."
        ),
        default=Population.PERSONNEL_ONLY,
    )
    people_parser.add_argument(
        "--sftp",
        action="store_true",
        help="Envoyer le fichier via SFTP sur le serveur gpuc2024.",
    )

    # Subparser pour les structures (structure)
    structure_parser = subparsers.add_parser(
        "structure", help="Gestion des structures."
    )
    structure_parser.add_argument(
        "action", choices=["generate"], help="Action à exécuter."
    )
    structure_parser.add_argument(
        "--sftp", action="store_true", help="Envoyer les fichiers via SFTP."
    )

    args = parser.parse_args()

    # Configuration des logs
    _init_logging(verbose=args.verbose, sftp_enabled=args.sftp)

    if args.command == "people" and args.action == "generate":
        logging.info("Exécution de la commande 'people generate'")
        status = TPeople.generate(population=args.population, sftp=args.sftp)
        print(status.description)

    elif args.command == "structure" and args.action == "generate":
        logging.debug("Exécution de la commande 'structure generate'")
        status = TStrucutre().generate(sftp=args.sftp)
        print(status.description)


if __name__ == "__main__":
    main()
