from dataclasses import dataclass
import logging
import re
import tempfile
import csv
from pathlib import Path

from gpuc.lib.ldap.ldapm import LdapmClient
from gpuc.lib.msftp import SFTPClient
from gpuc import FOLDER_STRUCTURE
from gpuc.config import GPUC_SETTINGS

logger = logging.getLogger(__name__)


@dataclass
class GenerateStatus:
    """Classe représentant le statut d'exécution de la fonction generate."""

    code: int

    @property
    def description(self) -> str:
        """Retourne une description fixe en fonction du code."""
        descriptions = {
            1: "Succès : fichiers générés et envoyés avec succès par SFTP.",
            2: "Succès : fichiers générés avec succès et sauvardés localement.",
            11: "Erreur : échec lors de l'envoi via SFTP.",
            12: "Erreur : échec lors de la génération des fichiers.",
        }
        return descriptions.get(self.code, "Erreur inconnue.")


@dataclass
class TStrucutre:
    """Classe pour la construction des fichiers d'affectation pour GPUC"""

    @staticmethod
    def _lenchem(infuo: str) -> int:
        """Récupère la profondeur d'une structure à partir de son code UO

        Args:
            infuo (str): L'information disponible dans l'attribut runUnivStrInfos de l'UO dans Ldap

        Returns:
            Un entier qui détermine la profondeur (le niveau) d'une UO

        Examples:
            >>> a = "[source={SIHAM}HUX0000000][chemin=Université/Extérieur]"
            >>> b = a + "[date_deb=19500101000000+0400][date_fin=29991231000000+0400][date_rnv=]"
            >>> TStrucutre._lenchem(b)
            1
        """
        # Motif regex pour la valeur de la clé "chemin"
        pattern = r"\[chemin=(.*?)\]"

        # Recherche de la correspondance dans le string
        match = re.search(pattern, infuo)

        # Vérification si une correspondance a été trouvée
        if match:
            # Récupération de la valeur de la clé "chemin"
            chemin_value = match.group(1)
            # Affichage de la valeur
            return len(chemin_value.split("/")) - 1
        return -1

    def generate(self, *, sftp: bool = False):
        """
        Génération des fichiers structures pour GPUC et envoi via SFTP si demandé.

        Args:
            sftp (bool): Si True, les fichiers générés seront envoyés via SFTP.
        """
        # Récupération des structures
        structures = LdapmClient().get_siham_uo()

        # Définition des fichiers temporaires et des en-têtes
        levels = {
            1: (
                "gpuc-structure-niveau1-",
                ["CODE_AFFECTATION", "NOM_AFFECTATION", "UID_DIRECTEUR"],
            ),
            2: (
                "gpuc-structure-niveau2-",
                [
                    "CODE_DEPARTEMENT",
                    "NOM_DEPARTEMENT",
                    "CODE_AFFECTATION",
                    "UID_DIRECTEUR",
                ],
            ),
            3: (
                "gpuc-structure-niveau3-",
                ["CDOE_EQUIPE", "NOM_EQUIPE", "CODE_PARENT", "UID_DIRECTEUR"],
            ),
            4: (
                "gpuc-structure-niveau4-",
                ["CODE_DIVISION", "NOM_DIVISION", "CODE_EQUIPE", "UID_DIRECTEUR"],
            ),
            5: (
                "gpuc-structure-niveau5-",
                [
                    "CODE_SOUS_DIVISION",
                    "NOM_SOUS_DIVISION",
                    "CODE_DIVISION",
                    "UID_DIRECTEUR",
                ],
            ),
        }

        temp_files = {}
        try:
            # Création des fichiers temporaires
            for level, (prefix, header) in levels.items():
                temp_file = tempfile.NamedTemporaryFile(
                    mode="w",
                    newline="",
                    encoding="utf-8",
                    delete=False,
                    prefix=prefix,
                    dir=FOLDER_STRUCTURE,
                )
                temp_files[level] = (temp_file, header)

            # Écriture des en-têtes et des données
            writers = {}
            for level, (temp_file, header) in temp_files.items():
                writers[level] = csv.writer(temp_file, delimiter=";")
                writers[level].writerow(header)

            for structure in structures:
                level = self._lenchem(structure.runUnivStrInfos)
                if level in writers:
                    if level == 1:
                        # On retire les codes UO parent pour correspondre au template
                        s = structure.to_gpuc()[:-2] + structure.to_gpuc()[-1:]
                        writers[level].writerow(s)
                    else:
                        writers[level].writerow(structure.to_gpuc())

            # Fermeture des fichiers
            for temp_file, _ in temp_files.values():
                temp_file.close()

            # Envoi des fichiers via SFTP si demandé
            if sftp:
                sftp_client = SFTPClient(
                    hostname=GPUC_SETTINGS.host,
                    port=GPUC_SETTINGS.port,
                    username=GPUC_SETTINGS.username,
                    password=GPUC_SETTINGS.password,
                    private_key_path=GPUC_SETTINGS.ssh_ppk,
                )
                sftp_client.connect()
                for level, (temp_file, _) in temp_files.items():
                    remote_path = (
                        f"{GPUC_SETTINGS.path}/TEST_affectation_niveau{level}.csv"
                    )
                    sftp_client.upload_file(remote_path, temp_file.name)
                sftp_client.disconnect()
                logger.debug("Suppression des fichiers temporaires")
                for temp_file, _ in temp_files.values():
                    try:
                        Path(temp_file.name).unlink()
                    except Exception as e:
                        logger.warning(
                            "Impossible de supprimer le fichier temporaire %s : %s",
                            temp_file.name,
                            e,
                        )

            rs = GenerateStatus(code=1 if sftp else 2)
            logger.info("Code retour pour la fonction **generate** : %s", rs.code)
            return rs

        except Exception as e:
            rs = GenerateStatus(code=11 if sftp else 12)
            logger.debug("Code retour pour la fonction **generate** : %s", rs.code)
            logger.exception(
                "Erreur lors de la génération ou de l'envoi des fichiers : %s", e
            )
            return rs
