import logging
from dataclasses import dataclass, field
from pathlib import Path
from typing import Optional, List

import paramiko

logger = logging.getLogger(__name__)


@dataclass
class SFTPClient:
    """
    Classe pour gérer les connexions SFTP et effectuer des opérations comme
    l'upload, le download et l'exécution de commandes distantes.

    Attributes:
        hostname (str): Adresse du serveur SFTP.
        port (int): Port du serveur SFTP.
        username (str): Nom d'utilisateur pour l'authentification.
        password (str): Mot de passe pour l'authentification.
        private_key_path (Optional[str]): Chemin vers la clé privée pour l'authentification.
    """

    hostname: str
    port: int
    username: str
    password: str
    private_key_path: Optional[str] = None
    ssh_client: paramiko.client.SSHClient = field(init=False, repr=False)

    def __post_init__(self):
        """Initialise le client SSH."""
        self.ssh_client = paramiko.SSHClient()

    def connect(self) -> None:
        """
        Établit une connexion SSH au serveur SFTP.

        Raises:`
            Exception: Si une erreur se produit lors de la connexion au serveur SFTP.
        """
        self.ssh_client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        try:
            if self.private_key_path:
                private_key = paramiko.RSAKey(filename=self.private_key_path)
                self.ssh_client.connect(
                    hostname=self.hostname,
                    port=self.port,
                    username=self.username,
                    pkey=private_key,
                    look_for_keys=False,
                )
            else:
                self.ssh_client.connect(
                    hostname=self.hostname,
                    port=self.port,
                    username=self.username,
                    password=self.password,
                    look_for_keys=False,
                )
            logger.info("Connexion établie avec %s:%s", self.hostname, self.port)
        except Exception as e:
            logger.exception("Erreur lors de la connexion au serveur SFTP : %s", e)
            raise

    def disconnect(self) -> None:
        """
        Ferme la connexion SSH au serveur SFTP.
        """
        logger.info("Déconnexion réussie de %s:%s", self.hostname, self.port)
        self.ssh_client.close()

    def get_list_of_files(self, remote_file_path: str) -> List[str]:
        """
        Récupère la liste des fichiers dans un répertoire distant.

        Args:
            remote_file_path (str): Chemin du répertoire distant.

        Returns:
            List[str]: Liste des fichiers dans le répertoire.

        Raise:
            Exception: Si une erreur se produit lors de la récupération des fichiers.
        """
        with self.ssh_client.open_sftp() as sftp_client:
            try:
                files = sftp_client.listdir(remote_file_path)
                logger.info("Fichiers dans %s : %s", remote_file_path, files)
                return files
            except Exception as e:
                logger.exception("Erreur lors de la récupération des fichiers : %s", e)
                raise

    def download_file(self, remote_file_path: str, local_file_path: Path) -> None:
        """
        Télécharge un fichier depuis le serveur SFTP.

        Args:
            remote_file_path (str): Chemin du fichier distant.
            local_file_path (Path): Chemin local où le fichier sera téléchargé.

        Raise:
            FileNotFoundError: Si le fichier distant n'est pas trouvé.
            Exception: Si une erreur se produit lors du téléchargement du fichier.
        """
        local_file_path = Path(local_file_path)
        with self.ssh_client.open_sftp() as sftp_client:
            try:
                sftp_client.get(remote_file_path, str(local_file_path))
                logger.info(
                    "Fichier téléchargé : %s -> %s", remote_file_path, local_file_path
                )
            except FileNotFoundError:
                logger.error("Fichier introuvable : %s", remote_file_path)
                raise
            except Exception as e:
                logger.exception("Erreur lors du téléchargement du fichier : %s", e)
                raise

    def upload_file(self, remote_file_path: str, local_file_path: Path) -> None:
        """
        Envoie un fichier vers le serveur SFTP.

        Args:
            remote_file_path (str): Chemin distant où le fichier sera envoyé.
            local_file_path (Path): Chemin local du fichier à envoyer.

        Raise:
            FileNotFoundError: Si le fichier local n'est pas trouvé.
            Exception: Si une erreur se produit lors de l'envoi du fichier.
        """
        local_file_path = Path(local_file_path)
        with self.ssh_client.open_sftp() as sftp_client:
            try:
                sftp_client.put(str(local_file_path), remote_file_path)
                logger.info(
                    "Fichier envoyé : %s -> %s", local_file_path, remote_file_path
                )
            except FileNotFoundError:
                logger.error("Fichier local introuvable : %s", local_file_path)
                raise
            except Exception as e:
                logger.exception("Erreur lors de l'envoi du fichier : %s", e)
                raise

    def execute_command(self, command: str) -> None:
        """
        Exécute une commande sur le serveur distant.

        Args:
            command (str): Commande à exécuter.

        Raise:
            Exception: Si une erreur se produit lors de l'exécution de la commande.
        """
        try:
            stdin, stdout, stderr = self.ssh_client.exec_command(command)
            logger.info("Résultat de la commande : %s", stdout.read().decode().strip())
            error = stderr.read().decode().strip()
            if error:
                logger.error("Erreur lors de l'exécution de la commande : %s", error)
        except Exception as e:
            logger.exception("Erreur lors de l'exécution de la commande : %s", e)
            raise
